@@ -5,8 +5,10 @@ sdf <- readNAEP(system.file("extdata/data", "M36NT2PM.dat", package = "NAEPprime
 summary2(sdf, "composite")
 
 \dontrun{
-# print out summary of weighted statistics of a variable including omitted levels
+# print out summary of weighted statistics of a variable, including omitted levels
 summary2(sdf, "b017451", omittedLevels = FALSE)
+# make a crosstab
+summary2(sdf, c("b017451", "dsex"), omittedLevels = FALSE)
 
 # print out summary of unweighted statistics of a variable
 summary2(sdf, "composite", weightVar = NULL)
